library(dataresqc)
library(tidyr)
library(tibble)
library(zoo)
library(xlsx)

##################################
###          BOLOGNA
##################################


#create station code
name_folder <- strsplit(getwd(), "/")[[1]][8]
name_group <- strsplit(getwd(), "/")[[1]][6]
station_code <- paste(name_group, name_folder, sep = "_")

#define wind directions
directions <- c("N", "NbE", "NNE", "NEbN", "NE", "NEbE", "ENE", "EbN", "E", 
                "EbS", "ESE", "SEbE", "SE", "SEbS", "SSE", "SbE", "S",
                "SbW", "SSW", "SWbS", "SW", "SWbW", "WSW", "WbS", "W",
                "WbN", "WNW", "NWbW", "NW", "NWbN", "NNW", "NbW")

#read inventory
inventory <- read.xlsx("/scratch3/PALAEO-RA/DataRescue/Inventory_PALAEO-RA_digi.xls", sheetIndex=1, colTypes="character")



### SUBDAILY

#read metadata
metadata <- inventory[which(inventory$Code == station_code & 
                              inventory$Resolution == "subdaily" &
                              inventory$First.Year == "1782"), ]

#files
files <- c("Palatine-Society_T3_IT_Bologna_1782_subdaily.xls",
           "Palatine-Society_T3_IT_Bologna_1783_subdaily.xls",
           "Palatine-Society_T3_IT_Bologna_1784_subdaily.xls",
           "Palatine-Society_T3_IT_Bologna_1787_subdaily.xls",
           "Palatine-Society_T3_IT_Bologna_1788_subdaily.xls",
           "Palatine-Society_T3_IT_Bologna_1791_subdaily.xls",
           "Palatine-Society_T3_IT_Bologna_1792_subdaily.xls")

#extrapolation of atb from ta with lm
template <- list()
for (i in c(1:3,6:7)) {
  template[[i]] <- suppressWarnings(read.xlsx(paste0("../../3_Revised/Bologna/",files[i]),
                                              sheetIndex=1, startRow=8, header=FALSE)[,1:8])
}
template <- Reduce(rbind, template)
xmean <- round(aggregate(template[,c(7:8)], list(template[,1],template[,2],template[,3]), mean, na.rm=TRUE), 1)
xmean <- xmean[order(xmean$Group.1, xmean$Group.2, xmean$Group.3), ]
lm <- list()
for (i in c(1:12)) {
  j <- which(xmean[,2] == i)
  lm[[i]] <- lm(xmean[j,4]~xmean[j,5])
}

##loop on files
out <- list()
for (i in 1:length(files)) {
  
  #read data
  template <- read.xlsx(paste0("../../3_Revised/Bologna/",files[i]),
                        sheetIndex=1, startRow=8, header=FALSE)
  
  #time
  time <- as.numeric(template[,4])
  hour <- as.integer(time)
  minute <- as.integer((time - hour) * 60)
  dates <- as.Date(paste(template[,1], template[,2], template[,3], sep="-"))
  meta_time <- paste0("orig.date=", dates, "|orig.time=", template[,4])
  
  #initialize data frame
  out[[i]] <- data.frame(year = as.integer(template[,1]), 
                         month = as.integer(template[,2]), 
                         day = as.integer(template[,3]), 
                         hour = hour, minute = minute)
  
  #pressure
  if (!grepl("87|88", files[i])) {
    template[,5] <- na.locf0(template[,5])
    p_in <- as.numeric(template[,5]) + as.numeric(template[,6]) / 12
    out[[i]]$p <- round(convert_pressure(p_in, 27.07, 
                                         as.numeric(metadata$Latitude), 
                                         as.numeric(metadata$Altitude),
                                         as.numeric(as.character(template[,7]))*1.25), 
                        1)
    i_miss <- which(is.na(as.numeric(template[,7])))
    out[[i]]$p[i_miss] <- round(convert_pressure(p_in[i_miss], 27.07, 
                                                 as.numeric(metadata$Latitude), 
                                                 as.numeric(metadata$Altitude)),
                                1)
    out[[i]]$p_meta <- paste0(meta_time, "|orig=", template[,5], ".", template[,6],
                              "Pin|atb=", template[,7], "R")
    out[[i]]$p_meta[i_miss] <- paste0(meta_time[i_miss], "|orig=", template[i_miss,5], ".", 
                                      template[i_miss,6], "Pin|atb=", template[i_miss,7], "|PTC=N")
  } else {
    out[[i]]$p <- NA
    out[[i]]$p_meta <- ""
  }
  
  #temperature
  if (!grepl("87|88", files[i])) {
    out[[i]]$ta <- round(as.numeric(as.character(template[,8])) * 1.25, 1)
    out[[i]]$ta_meta <- paste0(meta_time, "|orig=", template[,8], "R")
  } else {
    out[[i]]$ta <- NA
    out[[i]]$ta_meta <- ""
  }

  #wind direction
  j <- ifelse(!grepl("87|88", files[i]), "X12", 5)
  directions <- gsub("E", "O", directions)
  out[[i]]$dd <- round(11.25 * (match(as.character(template[,j]), directions) - 1), 0)
  out[[i]]$dd_meta <- paste0(meta_time, "|orig=", template[,j])

  #precipitation
  if (!grepl("87|88|92", files[i])) {
    template$X14[is.na(template$X14)] <- 0
    #template[which(time!=14),"X14"] <- NA
    out[[i]]$rr <- round((template$X14/(64*12)) * 27.07, 1)
    out[[i]]$rr_meta <- paste0(meta_time, "|orig=", template$X14, "lin./64")
  } else {
    out[[i]]$rr <- NA
    out[[i]]$rr_meta <- ""
  }
}


#merge dataframes
out <- Reduce(rbind, out)
#plot(out$rr)

#write SEF files
write_sef(out[,c("year","month","day","hour","minute","p")],
          outpath = paste0("../../4_Formatted/", name_folder),
          variable = "p",
          cod = metadata$Code,
          nam = metadata$Station,
          lat = metadata$Latitude,
          lon = metadata$Longitude,
          alt = metadata$Altitude,
          sou = "PALAEO-RA",
          link = metadata$C3S.Link,
          units = "hPa",
          stat = "point",
          metaHead = paste0("Observer=", metadata$Observer, 
                            "|PGC=Y|PTC=Y|Instrument=Palatine Society"),
          meta = out$p_meta,
          period = 0,
          time_offset = as.numeric(as.character(metadata$Longitude)) * 12 / 180,
          keep_na = FALSE)

write_sef(out[,c("year","month","day","hour","minute","ta")],
          outpath = paste0("../../4_Formatted/", name_folder),
          variable = "ta",
          cod = metadata$Code,
          nam = metadata$Station,
          lat = metadata$Latitude,
          lon = metadata$Longitude,
          alt = metadata$Altitude,
          sou = "PALAEO-RA",
          link = metadata$C3S.Link,
          units = "C",
          stat = "point",
          metaHead = paste0("Observer=", metadata$Observer, "|Instrument=Palatine Society"),
          meta = out$ta_meta,
          period = 0,
          time_offset = as.numeric(as.character(metadata$Longitude)) * 12 / 180,
          keep_na = FALSE)

write_sef(out[,c("year","month","day","hour","minute","dd")],
          outpath = paste0("../../4_Formatted/", name_folder),
          variable = "dd",
          cod = metadata$Code,
          nam = metadata$Station,
          lat = metadata$Latitude,
          lon = metadata$Longitude,
          alt = metadata$Altitude,
          sou = "PALAEO-RA",
          link = metadata$C3S.Link,
          units = "degree",
          stat = "point",
          metaHead = paste0("Observer=", metadata$Observer),
          meta = out$dd_meta,
          period = 0,
          time_offset = as.numeric(as.character(metadata$Longitude)) * 12 / 180,
          keep_na = TRUE)

out <- out[which(out$hour==14), ]
write_sef(out[,c("year","month","day","hour","minute","rr")],
          outpath = paste0("../../4_Formatted/", name_folder),
          variable = "rr",
          cod = metadata$Code,
          nam = metadata$Station,
          lat = metadata$Latitude,
          lon = metadata$Longitude,
          alt = metadata$Altitude,
          sou = "PALAEO-RA",
          link = metadata$C3S.Link,
          units = "mm",
          stat = "sum",
          metaHead = paste0("Observer=", metadata$Observer),
          meta = out$rr_meta,
          period = "day",
          time_offset = as.numeric(as.character(metadata$Longitude)) * 12 / 180,
          keep_na = FALSE)



### DAILY

#read metadata
metadata <- inventory[which(inventory$Code == station_code & 
                              inventory$Resolution == "daily" &
                              inventory$First.Year == "1787"), ]
#files
files <- c("Palatine-Society_T3_IT_Bologna_1787_daily.xls",
           "Palatine-Society_T3_IT_Bologna_1788_daily.xls")

##loop on files
out <- list()
for (i in 1:length(files)) {
  
  #read data
  template <- read.xlsx(paste0("../../3_Revised/Bologna/",files[i]),
                        sheetIndex=1, startRow=8, header=FALSE)
  
  #time
  hour <- rep(NA, length(template[,1]))
  minute <- rep(NA, length(template[,1]))
  dates <- as.Date(paste(template[,1], template[,2], template[,3], sep="-"))
  meta_time <- paste0("orig.date=", dates)
  
  #initialize data frame
  out[[i]] <- data.frame(year = as.integer(template[,1]), 
                         month = as.integer(template[,2]), 
                         day = as.integer(template[,3]),
                         hour = hour, minute = minute)
  
  #pressure
  template[,4] <- na.locf(template[,4])
  p_in <- as.numeric(template[,4]) + as.numeric(template[,5]) / 12
  template$atb <- NA
  for (k in c(1:12)) {
    template$atb[which(template[,2] == k)] <- round(lm[[k]]$coefficients[1] +
      as.numeric(as.character(template$X8[which(template[,2] == k)])) * lm[[k]]$coefficients[2], 1)}
  out[[i]]$p <- round(convert_pressure(p_in, 27.07, 
                                       as.numeric(metadata$Latitude), 
                                       as.numeric(metadata$Altitude),
                                       template$atb*1.25), 1)
  out[[i]]$p_meta <- paste0(meta_time, "|orig=", template[,4], ".", template[,5],
                            "Pin", "|atb=", template$atb, "R (extrapolated from ta)")
  
  #temperature
  out[[i]]$ta <- round(as.numeric(as.character(template[,8])) * 1.25, 1)
  out[[i]]$ta_meta <- paste0(meta_time, "|orig=", template[,8], "R")
}


#merge dataframes
out <- Reduce(rbind, out)

#write SEF files
write_sef(out[,c("year","month","day","hour","minute","p")],
          outpath = paste0("../../4_Formatted/", name_folder),
          variable = "p",
          cod = metadata$Code,
          nam = metadata$Station,
          lat = metadata$Latitude,
          lon = metadata$Longitude,
          alt = metadata$Altitude,
          sou = "PALAEO-RA",
          link = metadata$C3S.Link,
          units = "hPa",
          stat = "mean",
          metaHead = paste0("Observer=", metadata$Observer, "|PGC=Y|PTC=Y|Instrument=Palatine Society"),
          meta = out$p_meta,
          period = "day",
          time_offset = as.numeric(as.character(metadata$Longitude)) * 12 / 180,
          keep_na = FALSE,
          note = "daily")

write_sef(out[,c("year","month","day","hour","minute","ta")],
          outpath = paste0("../../4_Formatted/", name_folder),
          variable = "ta",
          cod = metadata$Code,
          nam = metadata$Station,
          lat = metadata$Latitude,
          lon = metadata$Longitude,
          alt = metadata$Altitude,
          sou = "PALAEO-RA",
          link = metadata$C3S.Link,
          units = "C",
          stat = "mean",
          metaHead = paste0("Observer=", metadata$Observer),
          meta = out$ta_meta,
          period = "day",
          time_offset = as.numeric(as.character(metadata$Longitude)) * 12 / 180,
          keep_na = TRUE,
          note = "daily")



### MONTHLY

#read metadata
metadata <- inventory[which(inventory$Code == station_code & 
                              inventory$Resolution == "monthly" &
                              inventory$First.Year == "1787"), ]
#files
files <- c("Palatine-Society_T3_IT_Bologna_1789-1790_monthly.xls",
           "Palatine-Society_T3_IT_Bologna_1787-1792_monthly-prec.xls")

#read data
template <- read.xlsx(paste0("../../3_Revised/Bologna/",files[1]),
                      sheetIndex=1, startRow=8, header=FALSE)

#initialize data frame
out <- data.frame(year = as.integer(template[,1]), 
                  month = as.integer(template[,2]), 
                  day = rep(NA, nrow(template)), 
                  hour = rep(NA, nrow(template)),
                  minute = rep(NA, nrow(template)))

template$atb <- NA
#pressure
p_in_max <- as.numeric(template[,3]) + as.numeric(template[,4]) / 12
out$pmax <- round(convert_pressure(p_in_max, 27.07,
                                   as.numeric(metadata$Latitude), 
                                   as.numeric(as.character(metadata$Altitude))), 1)
out$pmax_meta <- paste0("orig=", template[,3], ".", template[,4], "Pin")
#out$day <- c(7,NA,25,22,10,13,9,5,9,27,NA,9,10,19,13,22,25,21,20,16,21,17,NA,NA)
write_sef(out[,c("year","month","day","hour","minute","pmax")],
          outpath = paste0("../../4_Formatted/", name_folder),
          variable = "p",
          cod = metadata$Code,
          nam = metadata$Station,
          lat = metadata$Latitude,
          lon = metadata$Longitude,
          alt = metadata$Altitude,
          sou = "PALAEO-RA",
          link = metadata$C3S.Link,
          units = "hPa",
          stat = "maximum",
          metaHead = paste0("Observer=", metadata$Observer, 
                            "|PGC=Y|PTC=N|Instrument=Palatine Society"),
          meta = out$pmax_meta,
          period = "month",
          time_offset = as.numeric(as.character(metadata$Longitude)) * 12 / 180,
          keep_na = FALSE,
          note = "monthly-max")
p_in_min <- as.numeric(template[,6]) + as.numeric(template[,7]) / 12
out$pmin <- round(convert_pressure(p_in_min, 27.07,
                                   as.numeric(metadata$Latitude), 
                                   as.numeric(as.character(metadata$Altitude))), 1)
out$pmin_meta <- paste0("orig=", template[,6], ".", template[,7], "Pin")
#out$day <- c(10,NA,12,25,5,5,19,22,17,5,NA,16,30,10,31,26,7,13,4,3,3,29,NA,NA)
write_sef(out[,c("year","month","day","hour","minute","pmin")],
          outpath = paste0("../../4_Formatted/", name_folder),
          variable = "p",
          cod = metadata$Code,
          nam = metadata$Station,
          lat = metadata$Latitude,
          lon = metadata$Longitude,
          alt = metadata$Altitude,
          sou = "PALAEO-RA",
          link = metadata$C3S.Link,
          units = "hPa",
          stat = "minimum",
          metaHead = paste0("Observer=", metadata$Observer, 
                            "|PGC=Y|PTC=N|Instrument=Palatine Society"),
          meta = out$pmin_meta,
          period = "month",
          time_offset = as.numeric(as.character(metadata$Longitude)) * 12 / 180,
          keep_na = FALSE,
          note = "monthly-min")
for (k in c(1:12)) {
  template$atb[which(template[,2] == k)] <- round(lm[[k]]$coefficients[1] +
       template$X15[which(template[,2] == k)] * lm[[k]]$coefficients[2], 1)}
p_in_med <- as.numeric(template[,9]) + as.numeric(template[,10]) / 12
out$pmed <- round(convert_pressure(p_in_med, 27.07,
                                   as.numeric(metadata$Latitude), 
                                   as.numeric(as.character(metadata$Altitude)),
                                   template$atb*1.25), 1)
out$pmed_meta <- paste0("orig=", template[,9], ".", template[,10], "Pin", "|atb=", template$atb, "R (extrapolated from ta)")
#out$day <- rep(NA, 24)
write_sef(out[,c("year","month","day","hour","minute","pmed")],
          outpath = paste0("../../4_Formatted/", name_folder),
          variable = "p",
          cod = metadata$Code,
          nam = metadata$Station,
          lat = metadata$Latitude,
          lon = metadata$Longitude,
          alt = metadata$Altitude,
          sou = "PALAEO-RA",
          link = metadata$C3S.Link,
          units = "hPa",
          stat = "mid_range",
          metaHead = paste0("Observer=", metadata$Observer, 
                            "|PGC=Y|PTC=Y|Instrument=Palatine Society"),
          meta = out$pmed_meta,
          period = "month",
          time_offset = as.numeric(as.character(metadata$Longitude)) * 12 / 180,
          keep_na = FALSE,
          note = "monthly-midrange")

#temperature
out$tmax <- round(template[,11]*1.25, 1)
out$tmax_meta <- paste0("orig=", template[,11], "R")
#out$day <- c(30,NA,24,14,14,21,4,7,5,1,NA,5,2,28,27,30,29,23,21,8,18,5,NA,NA)
write_sef(out[,c("year","month","day","hour","minute","tmax")],
          outpath = paste0("../../4_Formatted/", name_folder),
          variable = "ta",
          cod = metadata$Code,
          nam = metadata$Station,
          lat = metadata$Latitude,
          lon = metadata$Longitude,
          alt = metadata$Altitude,
          sou = "PALAEO-RA",
          link = metadata$C3S.Link,
          units = "C",
          stat = "maximum",
          metaHead = paste0("Observer=", metadata$Observer, " |Instrument=Palatine Society"),
          meta = out$tmax_meta,
          period = "month",
          time_offset = as.numeric(as.character(metadata$Longitude)) * 12 / 180,
          keep_na = FALSE,
          note = "monthly-max")
out$tmin <- round(template[,13]*1.25, 1)
out$tmin_meta <- paste0("orig=", template[,13], "R")
#out$day <- c(1,NA,10,1,1,8,1,23,26,28,NA,20,29,2,6,3,8,19,9,19,28,31,NA,NA)
write_sef(out[,c("year","month","day","hour","minute","tmin")],
          outpath = paste0("../../4_Formatted/", name_folder),
          variable = "ta",
          cod = metadata$Code,
          nam = metadata$Station,
          lat = metadata$Latitude,
          lon = metadata$Longitude,
          alt = metadata$Altitude,
          sou = "PALAEO-RA",
          link = metadata$C3S.Link,
          units = "C",
          stat = "minimum",
          metaHead = paste0("Observer=", metadata$Observer, " |Instrument=Palatine Society"),
          meta = out$tmin_meta,
          period = "month",
          time_offset = as.numeric(as.character(metadata$Longitude)) * 12 / 180,
          keep_na = FALSE,
          note = "monthly-min")
out$tmed <- round(template[,15]*1.25, 1)
out$tmed_meta <- paste0("orig=", template[,15], "R")
#out$day <- rep(NA, 24)
write_sef(out[,c("year","month","day","hour","minute","tmed")],
          outpath = paste0("../../4_Formatted/", name_folder),
          variable = "ta",
          cod = metadata$Code,
          nam = metadata$Station,
          lat = metadata$Latitude,
          lon = metadata$Longitude,
          alt = metadata$Altitude,
          sou = "PALAEO-RA",
          link = metadata$C3S.Link,
          units = "C",
          stat = "mid_range",
          metaHead = paste0("Observer=", metadata$Observer, " |Instrument=Palatine Society"),
          meta = out$tmed_meta,
          period = "month",
          time_offset = as.numeric(as.character(metadata$Longitude)) * 12 / 180,
          keep_na = FALSE,
          note = "monthly-midrange")

#precipitation
out <- list()
for (i in 1:length(files)) {
  
  #read data
  template <- read.xlsx(paste0("../../3_Revised/Bologna/",files[i]),
                        sheetIndex=1, startRow=8, header=FALSE)
  #initialize data frame
  out[[i]] <- data.frame(year = as.integer(template[,1]), 
                         month = as.integer(template[,2]), 
                         day = rep(NA, nrow(template)), 
                         hour = rep(NA, nrow(template)),
                         minute = rep(NA, nrow(template)))
  
  if (!grepl("90", files[i])) {
    out[[i]]$rr <- round((as.numeric(as.character(template[,3]))/(64*12)) * 27.07, 1)
    out[[i]]$rr_meta <- paste0("orig=", template[,3], "lin./64")
  } else {
    out[[i]]$rr <- round((as.numeric(as.character(template[,20]))/(64*12)) * 27.07, 1)
    out[[i]]$rr_meta <- paste0("orig=", template[,20], "lin./64")
  }
}

#merge dataframes
out <- Reduce(rbind, out)
out <- rbind(out[25:48,],out[1:24,],out[49:60,])

write_sef(out[,c("year","month","day","hour","minute","rr")],
          outpath = paste0("../../4_Formatted/", name_folder),
          variable = "rr",
          cod = metadata$Code,
          nam = metadata$Station,
          lat = metadata$Latitude,
          lon = metadata$Longitude,
          alt = metadata$Altitude,
          sou = "PALAEO-RA",
          link = metadata$C3S.Link,
          units = "mm",
          stat = "mean",
          metaHead = paste0("Observer=", metadata$Observer, " |Instrument=Palatine Society"),
          meta = out$rr_meta,
          period = "month",
          time_offset = as.numeric(as.character(metadata$Longitude)) * 12 / 180,
          keep_na = FALSE,
          note = "monthly")



#check SEF files
for (f in list.files(paste0("../../4_Formatted/", name_folder), full.names=TRUE)) {
  print(f)
  check_sef(f)
}
