## NOTE: monthly precipitation was not formatted (unknown units)

library(dataresqc)
library(tidyr)
library(zoo)
library(readxl)

##################################
###          GODTHAAB
##################################

#create station code
#setwd("/scratch3/PALEO-RA/DataRescue/Projects/Palatine-Society/3_Revised/Godthaab")
name_folder <- "Godthaab"
name_group <- "Palatine-Society"
station_code <- paste(name_group, name_folder, sep = "_")

#read inventory
inventory <- as.data.frame(read_xls("/scratch3/PALAEO-RA/DataRescue/Inventory_PALAEO-RA_digi.xls", sheet=1, col_names=T, col_types="text"))

#define wind directions
directions <- c("N", "NbE", "NNE", "NEbN", "NE", "NEbE", "ENE", "EbN", "E", 
                "EbS", "ESE", "SEbE", "SE", "SEbS", "SSE", "SbE", "S",
                "SbW", "SSW", "SWbS", "SW", "SWbW", "WSW", "WbS", "W",
                "WbN", "WNW", "NWbW", "NW", "NWbN", "NNW", "NbW")


### SUBDAILY

#read metadata
metadata <- inventory[which(inventory$Code == station_code & 
                              inventory$Resolution == "subdaily"), ]

#files
files <- "Palatine-Society_T3_GL_Godthaab_1786-1787_subdaily.xlsx"

#read data
template <- data.frame(read_xlsx(paste0("../../3_Revised/Godthaab/", files), sheet=1, skip=6, col_names=T))
                        
#time
hour <- as.integer(template[,4])
minute <- as.integer((template[,4] - hour) * 60)
date <- as.Date(paste(template[,1], sprintf("%02d", template[,2]), 
                      sprintf("%02d", template[,3]), sep="-"))
meta_time <- paste0("orig.date=", date, "|orig.time=", template[,4])
  
#initialize data frame
out <- data.frame(year = as.integer(template[,1]), 
                  month = as.integer(template[,2]), 
                  day = as.integer(template[,3]), 
                  hour = hour, minute = minute)

#wind direction
directions <- gsub("E", "O", directions)
out$dd <- round(11.25 * (match(as.character(template$dd), directions) - 1), 0)
out$dd_meta <- paste0(meta_time, "|orig=", template$dd)

#wind force
out$w <- as.numeric(as.character(template$w))
out$w_meta <- paste0(meta_time, "|orig=", as.character(template$w))

#write SEF files
write_sef(out[,c("year","month","day","hour","minute","dd")],
          outpath = paste0("../../4_Formatted/Godthaab/"),
          variable = "dd",
          cod = metadata$Code,
          nam = metadata$Station,
          lat = metadata$Latitude,
          lon = metadata$Longitude,
          alt = metadata$Altitude,
          sou = "PALAEO-RA",
          link = metadata$C3S.Link,
          units = "degree",
          stat = "point",
          metaHead = paste0("Observer=", metadata$Observer),
          meta = out$dd_meta,
          period = 0,
          time_offset = as.numeric(as.character(metadata$Longitude)) * 12 / 180,
          keep_na = TRUE)

write_sef(out[,c("year","month","day","hour","minute","w")],
          outpath = paste0("../../4_Formatted/Godthaab/"),
          variable = "w",
          cod = metadata$Code,
          nam = metadata$Station,
          lat = metadata$Latitude,
          lon = metadata$Longitude,
          alt = metadata$Altitude,
          sou = "PALAEO-RA",
          link = metadata$C3S.Link,
          units = "0-4",
          stat = "point",
          metaHead = paste0("Observer=", metadata$Observer),
          meta = out$w_meta,
          period = 0,
          time_offset = as.numeric(as.character(metadata$Longitude)) * 12 / 180,
          keep_na = FALSE)



# DAILY

#files
files <- "Palatine-Society_T3_GL_Godthaab_1787_daily.xlsx"

#read data
template <- data.frame(read_xlsx(paste0("../../3_Revised/Godthaab/", files), sheet=1, skip=7, col_names=T))

#date
date <- as.Date(paste(template[,1], sprintf("%02d", template[,2]), 
                      sprintf("%02d", template[,3]), sep="-"))
meta_time <- paste0("orig.date=", date)

#initialize data frame
out <- data.frame(year = as.integer(template[,1]), month = as.integer(template[,2]), 
                  day = as.integer(template[,3]), hour = rep(NA, 181), minute = rep(NA, 181))

#pressure
template[,4] <- na.locf(template[,4])
p_in <- as.numeric(template[,4]) + as.numeric(template[,5]) / 12
out$p <- round(convert_pressure(p_in, 27.07, as.numeric(metadata$Latitude), as.numeric(metadata$Altitude)), 1)
out$p_meta <- paste0(meta_time, "|orig=", template[,4], ".", template[,5], "Pin")

#temperature
out$ta <- round((template[,6] * 1.25), 1)
out$ta_meta <- paste0(meta_time, "|orig=", template[,6], "R")

#write SEF files
write_sef(out[,c("year","month","day","hour","minute","p")],
          outpath = paste0("../../4_Formatted/Godthaab/"),
          variable = "p",
          cod = metadata$Code,
          nam = metadata$Station,
          lat = metadata$Latitude,
          lon = metadata$Longitude,
          alt = metadata$Altitude,
          sou = "PALAEO-RA",
          link = metadata$C3S.Link,
          units = "hPa",
          stat = "mean",
          metaHead = paste0("Observer=", metadata$Observer, "|PGC=Y|PTC=N|Instrument=Palatine Society"),
          meta = out$p_meta,
          period = "day",
          keep_na = F)

write_sef(out[,c("year","month","day","hour","minute","ta")],
          outpath = paste0("../../4_Formatted/Godthaab/"),
          variable = "ta",
          cod = metadata$Code,
          nam = metadata$Station,
          lat = metadata$Latitude,
          lon = metadata$Longitude,
          alt = metadata$Altitude,
          sou = "PALAEO-RA",
          link = metadata$C3S.Link,
          units = "C",
          stat = "mean",
          metaHead = paste0("Observer=", metadata$Observer, "|Instrument=Palatine Society"),
          meta = out$ta_meta,
          period = "day",
          keep_na = F)



#check SEF files
for (f in list.files(paste0("../../4_Formatted/Godthaab/"), full.names=TRUE)) {
  print(f)
  check_sef(f)
}
