library(dataresqc)
library(tidyr)
library(tibble)
library(zoo)
library(xlsx)

##################################
###          MOSCOW
##################################

#function to fill missing values
na.lomf <- function(x) {
  na.lomf.0 <- function(x) {
    non.na.idx <- which(!is.na(x))
    if (is.na(x[1L])) {
      non.na.idx <- c(1L, non.na.idx)
    }
    rep.int(x[non.na.idx], diff(c(non.na.idx, length(x) + 1L)))
  }
  
  dim.len <- length(dim(x))
  
  if (dim.len == 0L) {
    na.lomf.0(x)
  } else {
    apply(x, dim.len, na.lomf.0)
  }
}

#create station code
#setwd("/scratch3/PALEO-RA/DataRescue/Projects/Palatine-Society/3_Revised/Moscow")
name_folder <- strsplit(getwd(), "/")[[1]][8]
name_group <- strsplit(getwd(), "/")[[1]][6]
station_code <- paste(name_group, name_folder, sep = "_")

#define wind directions
directions <- c("N", "NbE", "NNE", "NEbN", "NE", "NEbE", "ENE", "EbN", "E", 
                "EbS", "ESE", "SEbE", "SE", "SEbS", "SSE", "SbE", "S",
                "SbW", "SSW", "SWbS", "SW", "SWbW", "WSW", "WbS", "W",
                "WbN", "WNW", "NWbW", "NW", "NWbN", "NNW", "NbW")

#read inventory
inventory <- read.xlsx("/scratch3/PALAEO-RA/DataRescue/Inventory_PALAEO-RA_digi.xls", sheetIndex=1, colTypes="character")



### SUBDAILY

#read metadata
metadata <- inventory[which(inventory$Code == station_code & 
                              inventory$Resolution == "subdaily" &
                              inventory$First.Year == "1785"), ]

#files
files <- c("Palatine-Society_T3_RU_Moscow_1785_subdaily.xls",
           "Palatine-Society_T3_RU_Moscow_1786_subdaily.xls",
           "Palatine-Society_T3_RU_Moscow_1787_subdaily.xls",
           "Palatine-Society_T3_RU_Moscow_1788_subdaily.xls",
           "Palatine-Society_T3_RU_Moscow_1789_subdaily.xls",
           "Palatine-Society_T3_RU_Moscow_1791_subdaily.xls",
           "Palatine-Society_T3_RU_Moscow_1792_subdaily.xls")

##loop on files
out <- list()
for (i in 1:length(files)) {
  
  #read data
  template <- read.xlsx(paste0("../../3_Revised/Moscow/",files[i]),
                        sheetIndex=1, startRow=8, header=FALSE)
  
  #time
  time <- as.numeric(template[,ifelse(i>5,4,5)])
  hour <- as.integer(time)
  minute <- as.integer((time - hour) * 60)
  dates <- as.Date(paste(template[,1], template[,2], template[,3], sep="-"))
  meta_time <- paste0("orig.date=", dates, "|orig.time=", template[,ifelse(i>5,4,5)])
  
  #initialize data frame
  out[[i]] <- data.frame(year = as.integer(template[,1]), 
                         month = as.integer(template[,2]), 
                         day = as.integer(template[,3]), 
                         hour = hour, minute = minute)

  #pressure
  if (!grepl("87|91|92", files[i])) {
    template[,6] <- na.lomf(template[,6])
    p_in <- as.numeric(template[,6]) + as.numeric(template[,7]) / 24
    out[[i]]$p <- round(convert_pressure(p_in, 27.07, 
                                         as.numeric(metadata$Latitude), 
                                         as.numeric(metadata$Altitude)), 1)
    out[[i]]$p_meta <- paste0(meta_time, "|orig=", template[,6], ".", template[,7], "Pin")
  } else if (grepl("91|92", files[i])) {
    template[,5] <- na.lomf(template[,5])
    p_in <- as.numeric(template[,5]) + as.numeric(template[,6]) / 24
    out[[i]]$p <- round(convert_pressure(p_in, 27.07, 
                                         as.numeric(metadata$Latitude), 
                                         as.numeric(metadata$Altitude)), 1)
    out[[i]]$p_meta <- paste0(meta_time, "|orig=", template[,5], ".", template[,6], "Pin")
  } else {
    out[[i]]$p <- NA
    out[[i]]$p_meta <- ""
  }
  
  #temperature
  if (grepl("85|86", files[i])) {
    out[[i]]$ta <- round(100 - (as.numeric(as.character(template[,8])) * 2/3), 1) #Delisle
    out[[i]]$ta_meta <- paste0(meta_time, "|orig=", template[,8], "De")
  } else if (grepl("88|89", files[i])) {
    out[[i]]$ta <- round(as.numeric(as.character(template[,8])) * 1.25, 1) #Reaumur
    out[[i]]$ta_meta <- paste0(meta_time, "|orig=", template[,8], "R")
  } else if (grepl("91|92", files[i])) {
    out[[i]]$ta <- round(as.numeric(as.character(template[,7])) * 1.25, 1) #Reaumur
    out[[i]]$ta_meta <- paste0(meta_time, "|orig=", template[,7], "R")
  } else {
    out[[i]]$ta <- NA
    out[[i]]$ta_meta <- ""
  }

  #wind direction
  if (!grepl("91|92", files[i])) {
    j <- ifelse(!grepl("87", files[i]), 9, 6)
    directions <- gsub("E", "O", directions)
    out[[i]]$dd <- round(11.25 * (match(as.character(template[,j]), directions) - 1), 0)
    out[[i]]$dd_meta <- paste0(meta_time, "|orig=", template[,j])
  } else if (grepl("91|92", files[i])) {
    directions <- gsub("E", "O", directions)
    out[[i]]$dd <- round(11.25 * (match(as.character(template[,8]), directions) - 1), 0)
    out[[i]]$dd_meta <- paste0(meta_time, "|orig=", template[,8])
  }
}


#merge dataframes
out <- Reduce(rbind, out)

#write SEF files
write_sef(out[,c("year","month","day","hour","minute","p")],
          outpath = paste0("../../4_Formatted/", name_folder),
          variable = "p",
          cod = metadata$Code,
          nam = metadata$Station,
          lat = metadata$Latitude,
          lon = metadata$Longitude,
          alt = metadata$Altitude,
          sou = "PALAEO-RA",
          link = metadata$C3S.Link,
          units = "hPa",
          stat = "point",
          metaHead = paste0("Observer=", metadata$Observer, 
                            "|PGC=Y|PTC=N|Instrument=Palatine Society"),
          meta = out$p_meta,
          period = 0,
          time_offset = as.numeric(as.character(metadata$Longitude)) * 12 / 180,
          keep_na = FALSE)

#write SEF files
write_sef(out[,c("year","month","day","hour","minute","ta")],
          outpath = paste0("../../4_Formatted/", name_folder),
          variable = "ta",
          cod = metadata$Code,
          nam = metadata$Station,
          lat = metadata$Latitude,
          lon = metadata$Longitude,
          alt = metadata$Altitude,
          sou = "PALAEO-RA",
          link = metadata$C3S.Link,
          units = "C",
          stat = "point",
          metaHead = paste0("Observer=", metadata$Observer, "|Instrument=Palatine Society"),
          meta = out$ta_meta,
          period = 0,
          time_offset = as.numeric(as.character(metadata$Longitude)) * 12 / 180,
          keep_na = FALSE)

write_sef(out[,c("year","month","day","hour","minute","dd")],
          outpath = paste0("../../4_Formatted/", name_folder),
          variable = "dd",
          cod = metadata$Code,
          nam = metadata$Station,
          lat = metadata$Latitude,
          lon = metadata$Longitude,
          alt = metadata$Altitude,
          sou = "PALAEO-RA",
          link = metadata$C3S.Link,
          units = "degree",
          stat = "point",
          metaHead = paste0("Observer=", metadata$Observer),
          meta = out$dd_meta,
          period = 0,
          time_offset = as.numeric(as.character(metadata$Longitude)) * 12 / 180,
          keep_na = TRUE)



### MONTHLY

#read metadata
metadata <- inventory[which(inventory$Code == station_code & 
                              inventory$Resolution == "monthly" &
                              inventory$First.Year == "1783"), ]
#files
files <- c("Palatine-Society_T3_RU_Moscow_1783-1784_monthly.xls",
           "Palatine-Society_T3_RU_Moscow_1787_monthly.xls")


##loop on files
out <- list()
for (i in 1:length(files)) {

#read data
template <- read.xlsx(paste0("../../3_Revised/Moscow/",files[i]),
                      sheetIndex=1, startRow=8, header=FALSE)

#initialize data frame
out[[i]] <- data.frame(year = as.integer(template[,1]), 
                  month = as.integer(template[,2]), 
                  day = rep(NA, nrow(template)), 
                  hour = rep(NA, nrow(template)),
                  minute = rep(NA, nrow(template)))

#pressure
  if (grepl("83", files[i])) {
    p_in_max <- as.numeric(template[,3])
    out[[i]]$pmax <- round(convert_pressure(p_in_max, 25.4,
                                            as.numeric(metadata$Latitude), 
                                            as.numeric(as.character(metadata$Altitude))), 1)
    out[[i]]$pmax_meta <- paste0("orig=", template[,3], "Ein")
  } else if (grepl("87", files[i])) {
    p_in_max <- as.numeric(template[,3]) + as.numeric(template[,4]) / 24
    out[[i]]$pmax <- round(convert_pressure(p_in_max, 27.07,
                                            as.numeric(metadata$Latitude), 
                                            as.numeric(as.character(metadata$Altitude))), 1)
    out[[i]]$pmax_meta <- paste0("orig=", template[,3], ".", template[,4], "Pin")
  }

  if (grepl("83", files[i])) {
    p_in_min <- as.numeric(template[,6])
    out[[i]]$pmin <- round(convert_pressure(p_in_min, 25.4,
                                     as.numeric(metadata$Latitude), 
                                     as.numeric(as.character(metadata$Altitude))), 1)
    out[[i]]$pmin_meta <- paste0("orig=", template[,6], "Ein")
  } else if (grepl("87", files[i])) {
    p_in_min <- as.numeric(template[,6]) + as.numeric(template[,7]) / 24
    out[[i]]$pmin <- round(convert_pressure(p_in_min, 27.07,
                                     as.numeric(metadata$Latitude), 
                                     as.numeric(as.character(metadata$Altitude))), 1)
    out[[i]]$pmin_meta <- paste0("orig=", template[,6], ".", template[,7], "Pin")  
  }

  if (grepl("83", files[i])) {
    p_in_med <- as.numeric(template[,9])
    out[[i]]$pmed <- round(convert_pressure(p_in_med, 25.4,
                                     as.numeric(metadata$Latitude), 
                                     as.numeric(as.character(metadata$Altitude))), 1)
    out[[i]]$pmed_meta <- paste0("orig=", template[,9], "Ein")
  } else if (grepl("87", files[i])) {
    p_in_med <- as.numeric(template[,9]) + as.numeric(template[,10]) / 24
    out[[i]]$pmed <- round(convert_pressure(p_in_med, 27.07,
                                     as.numeric(metadata$Latitude), 
                                     as.numeric(as.character(metadata$Altitude))), 1)
    out[[i]]$pmed_meta <- paste0("orig=", template[,9], ".", template[,10], "Pin")
  }

#temperature
  if (grepl("83", files[i])) {
    out[[i]]$tmax <- round(100 - (as.numeric(as.character(template[,10])) * 2/3), 1)
    out[[i]]$tmax_meta <- paste0("orig=", template[,10], "De")
  } else if (grepl("87", files[i])) {
    out[[i]]$tmax <- round(100 - (as.numeric(as.character(template[,11])) * 2/3), 1)
    out[[i]]$tmax_meta <- paste0("orig=", template[,11], "De")
  }

  out[[i]]$tmin <- round(100 - (as.numeric(as.character(template[,13])) * 2/3), 1)
  out[[i]]$tmin_meta <- paste0("orig=", template[,13], "De")
  
  if (grepl("83", files[i])) {
    out[[i]]$tmed <- round(100 - (as.numeric(as.character(template[,16])) * 2/3), 1)
    out[[i]]$tmed_meta <- paste0("orig=", template[,16], "De")
  } else if (grepl("87", files[i])) {
    out[[i]]$tmed <- round(100 - (as.numeric(as.character(template[,15])) * 2/3), 1)
    out[[i]]$tmed_meta <- paste0("orig=", template[,15], "De")
  }

#number of rainy days
  if (grepl("83", files[i])) {
    out[[i]]$nr <- template[,17]
    out[[i]]$nr_meta <- paste0("orig=", template[,17])
  } else {
    out[[i]]$nr <- NA
    out[[i]]$nr_meta <- ""
  }
  
}


#merge dataframes
out <- Reduce(rbind, out)

#write SEF files
#out$day <- c(12,27,1,19,27,13,27,6,16,NA,24,10,28,3,25,11,23,12,
#             31,3,23,NA,15,27,17,27,1,2,1,27,20,27,24,1,24,21) # pmax days
write_sef(out[,c("year","month","day","hour","minute","pmax")],
          outpath = paste0("../../4_Formatted/", name_folder),
          variable = "p",
          cod = metadata$Code,
          nam = metadata$Station,
          lat = metadata$Latitude,
          lon = metadata$Longitude,
          alt = metadata$Altitude,
          sou = "PALAEO-RA",
          link = metadata$C3S.Link,
          units = "hPa",
          stat = "maximum",
          metaHead = paste0("Observer=", metadata$Observer, 
                            "|PGC=Y|PTC=N|Instrument=Palatine Society"),
          meta = out$pmax_meta,
          period = "month",
          time_offset = as.numeric(as.character(metadata$Longitude)) * 12 / 180,
          keep_na = FALSE,
          note = "monthly-max")
#out$day <- c(1,12,26,15,9,19,21,23,19,NA,9,17,5,18,29,24,7,1,
#             10,31,2,NA,13,12,18,8,7,6,6,12,2,20,4,17,10,30) # pmin days
write_sef(out[,c("year","month","day","hour","minute","pmin")],
          outpath = paste0("../../4_Formatted/", name_folder),
          variable = "p",
          cod = metadata$Code,
          nam = metadata$Station,
          lat = metadata$Latitude,
          lon = metadata$Longitude,
          alt = metadata$Altitude,
          sou = "PALAEO-RA",
          link = metadata$C3S.Link,
          units = "hPa",
          stat = "minimum",
          metaHead = paste0("Observer=", metadata$Observer, 
                            "|PGC=Y|PTC=N|Instrument=Palatine Society"),
          meta = out$pmin_meta,
          period = "month",
          time_offset = as.numeric(as.character(metadata$Longitude)) * 12 / 180,
          keep_na = FALSE,
          note = "monthly-min")
#out$day <- rep(NA, 36)
write_sef(out[!grepl("1787", out$year),c("year","month","day","hour","minute","pmed")],
          outpath = paste0("../../4_Formatted/", name_folder),
          variable = "p",
          cod = metadata$Code,
          nam = metadata$Station,
          lat = metadata$Latitude,
          lon = metadata$Longitude,
          alt = metadata$Altitude,
          sou = "PALAEO-RA",
          link = metadata$C3S.Link,
          units = "hPa",
          stat = "mean",
          metaHead = paste0("Observer=", metadata$Observer, 
                            "|PGC=Y|PTC=N|Instrument=Palatine Society"),
          meta = out$pmed_meta,
          period = "month",
          time_offset = as.numeric(as.character(metadata$Longitude)) * 12 / 180,
          keep_na = FALSE,
          note = "monthly-med")
write_sef(out[grepl("1787", out$year),c("year","month","day","hour","minute","pmed")],
          outpath = paste0("../../4_Formatted/", name_folder),
          variable = "p",
          cod = metadata$Code,
          nam = metadata$Station,
          lat = metadata$Latitude,
          lon = metadata$Longitude,
          alt = metadata$Altitude,
          sou = "PALAEO-RA",
          link = metadata$C3S.Link,
          units = "hPa",
          stat = "mid_range",
          metaHead = paste0("Observer=", metadata$Observer, 
                            "|PGC=Y|PTC=N|Instrument=Palatine Society"),
          meta = out$pmed_meta,
          period = "month",
          time_offset = as.numeric(as.character(metadata$Longitude)) * 12 / 180,
          keep_na = FALSE,
          note = "monthly-midrange")

#temperature
#out$day <- c(21,2,17,28,18,18,20,7,3,1,21,6,8,15,30,22,25,14,
#             21,1,12,NA,3,4,22,21,13,27,24,29,22,3,18,1,8,NA) # tmax days
write_sef(out[,c("year","month","day","hour","minute","tmax")],
          outpath = paste0("../../4_Formatted/", name_folder),
          variable = "ta",
          cod = metadata$Code,
          nam = metadata$Station,
          lat = metadata$Latitude,
          lon = metadata$Longitude,
          alt = metadata$Altitude,
          sou = "PALAEO-RA",
          link = metadata$C3S.Link,
          units = "C",
          stat = "maximum",
          metaHead = paste0("Observer=", metadata$Observer, " |Instrument=Palatine Society"),
          meta = out$tmax_meta,
          period = "month",
          time_offset = as.numeric(as.character(metadata$Longitude)) * 12 / 180,
          keep_na = FALSE,
          note = "monthly-max")
#out$day <- c(13,28,12,1,11,21,9,22,18,28,19,14,27,5,14,2,30,26,
#             12,31,22,NA,15,18,9,26,11,2,11,6,29,31,10,2,25,1) # tmin days
write_sef(out[,c("year","month","day","hour","minute","tmin")],
          outpath = paste0("../../4_Formatted/", name_folder),
          variable = "ta",
          cod = metadata$Code,
          nam = metadata$Station,
          lat = metadata$Latitude,
          lon = metadata$Longitude,
          alt = metadata$Altitude,
          sou = "PALAEO-RA",
          link = metadata$C3S.Link,
          units = "C",
          stat = "minimum",
          metaHead = paste0("Observer=", metadata$Observer, " |Instrument=Palatine Society"),
          meta = out$tmin_meta,
          period = "month",
          time_offset = as.numeric(as.character(metadata$Longitude)) * 12 / 180,
          keep_na = FALSE,
          note = "monthly-min")
#out$day <- rep(NA, 36)
write_sef(out[!grepl("1787", out$year),c("year","month","day","hour","minute","tmed")],
          outpath = paste0("../../4_Formatted/", name_folder),
          variable = "ta",
          cod = metadata$Code,
          nam = metadata$Station,
          lat = metadata$Latitude,
          lon = metadata$Longitude,
          alt = metadata$Altitude,
          sou = "PALAEO-RA",
          link = metadata$C3S.Link,
          units = "C",
          stat = "mean",
          metaHead = paste0("Observer=", metadata$Observer, " |Instrument=Palatine Society"),
          meta = out$tmed_meta,
          period = "month",
          time_offset = as.numeric(as.character(metadata$Longitude)) * 12 / 180,
          keep_na = FALSE,
          note = "monthly-med")
write_sef(out[grepl("1787", out$year),c("year","month","day","hour","minute","tmed")],
          outpath = paste0("../../4_Formatted/", name_folder),
          variable = "ta",
          cod = metadata$Code,
          nam = metadata$Station,
          lat = metadata$Latitude,
          lon = metadata$Longitude,
          alt = metadata$Altitude,
          sou = "PALAEO-RA",
          link = metadata$C3S.Link,
          units = "C",
          stat = "mid_range",
          metaHead = paste0("Observer=", metadata$Observer, " |Instrument=Palatine Society"),
          meta = out$tmed_meta,
          period = "month",
          time_offset = as.numeric(as.character(metadata$Longitude)) * 12 / 180,
          keep_na = FALSE,
          note = "monthly-midrange")

#number of rainy days
write_sef(out[1:24,c("year","month","day","hour","minute","nr")],
          outpath = paste0("../../4_Formatted/", name_folder),
          variable = "nr",
          cod = metadata$Code,
          nam = metadata$Station,
          lat = metadata$Latitude,
          lon = metadata$Longitude,
          alt = metadata$Altitude,
          sou = "PALAEO-RA",
          link = metadata$C3S.Link,
          units = "count",
          stat = "sum",
          metaHead = paste0("Observer=", metadata$Observer),
          meta = out$nr_meta[1:24],
          period = "month",
          time_offset = as.numeric(as.character(metadata$Longitude)) * 12 / 180,
          keep_na = TRUE,
          note = "monthly")




#check SEF files
for (f in list.files(paste0("../../4_Formatted/", name_folder), full.names=TRUE)) {
  print(f)
  check_sef(f)
}
