library(dataresqc)
library(tidyr)
library(tibble)
library(zoo)
library(xlsx)

##################################
###          PRAGUE
##################################


#create station code
#setwd("/scratch3/PALEO-RA/DataRescue/Projects/Palatine-Society/3_Revised/Prague")
name_folder <- strsplit(getwd(), "/")[[1]][8]
name_group <- strsplit(getwd(), "/")[[1]][6]
station_code <- paste(name_group, name_folder, sep = "_")

#define wind directions
directions <- c("N", "NbE", "NNE", "NEbN", "NE", "NEbE", "ENE", "EbN", "E", 
                "EbS", "ESE", "SEbE", "SE", "SEbS", "SSE", "SbE", "S",
                "SbW", "SSW", "SWbS", "SW", "SWbW", "WSW", "WbS", "W",
                "WbN", "WNW", "NWbW", "NW", "NWbN", "NNW", "NbW")

#read inventory
inventory <- read.xlsx("/scratch3/PALAEO-RA/DataRescue/Inventory_PALAEO-RA_digi.xls", sheetIndex=1, colTypes="character")



### SUBDAILY

#read metadata
metadata <- inventory[which(inventory$Code == station_code & 
                              inventory$Resolution == "subdaily" &
                              inventory$First.Year == "1781"), ]

#files
# digitised by MeteoFrance within CHEDAR Project
files <- "Palatine-Society_T3_CZ_Prague_1781-1791_subdaily.xlsx"

#read data
template <- read.xlsx(paste0("../../3_Revised/Prague/",files),
                      sheetIndex=1, startRow=5, header=FALSE)

#extrapolation of atb from ta with lm
xmean <- round(aggregate(template[,c(8,10)], list(template[,1],template[,2],template[,3]), mean, na.rm=TRUE), 1)
xmean <- xmean[order(xmean$Group.1, xmean$Group.2, xmean$Group.3), ]
lm <- list()
for (i in c(1:12)) {
  j <- which(xmean[,2] == i)
  lm[[i]] <- lm(xmean[j,4]~xmean[j,5])
}

#time
time <- as.numeric(template[,4])
hour <- as.integer(time)
minute <- as.integer((time - hour) * 60)
dates <- as.Date(paste(template[,1], template[,2], template[,3], sep="-"))
meta_time <- paste0("orig.date=", dates, "|orig.time=", template[,4])

#initialize data frame
out <- data.frame(year = as.integer(template[,1]), 
                  month = as.integer(template[,2]), 
                  day = as.integer(template[,3]), 
                  hour = hour, minute = minute)

#pressure
template[,5] <- na.locf(template[,5])
p_in <- as.numeric(template[,5]) + as.numeric(template[,6]) / 12
out$p <- round(convert_pressure(p_in, 27.07, 
                                as.numeric(metadata$Latitude), 
                                as.numeric(metadata$Altitude),
                                as.numeric(template[,8])*1.25), 
               1)
i_miss <- which(is.na(as.numeric(template[,8])))
out$p[i_miss] <- round(convert_pressure(p_in[i_miss], 27.07, 
                                        as.numeric(metadata$Latitude), 
                                        as.numeric(metadata$Altitude)),
                       1)
out$p_meta <- paste0(meta_time, "|orig=", template[,5], ".", template[,6],
                     "Pin|atb=", template[,8], "R")
out$p_meta[i_miss] <- paste0(meta_time[i_miss], "|orig=", template[i_miss,5], ".", 
                             template[i_miss,6], "Pin|atb=", template[i_miss,8], "|PTC=N")

#temperature  
out$ta <- round(as.numeric(template[,10]) * 1.25, 1)
out$ta_meta <- paste0(meta_time, "|orig=", template[,10], "R")

#wind direction
directions <- gsub("E", "O", directions)
template <- template %>% separate(14, into = c("char", "num"), sep = "(?<=[a-zA-Z])(?=[0-9])")
out$dd <- round(11.25 * (match(template[,14], directions) - 1), 0)
out$dd_meta <- paste0(meta_time, "|orig=", template[,14])

#wind force
out$w <- as.integer(template[,15])
out$w_meta <- paste0(meta_time, "|orig=", template[,15])



#write SEF files
write_sef(out[,c("year","month","day","hour","minute","p")],
          outpath = paste0("../../4_Formatted/", name_folder),
          variable = "p",
          cod = metadata$Code,
          nam = metadata$Station,
          lat = metadata$Latitude,
          lon = metadata$Longitude,
          alt = metadata$Altitude,
          sou = "PALAEO-RA",
          link = metadata$C3S.Link,
          units = "hPa",
          stat = "point",
          metaHead = paste0("Observer=", metadata$Observer, 
                            "|PGC=Y|PTC=Y|Instrument=Palatine Society"),
          meta = out$p_meta,
          period = 0,
          time_offset = as.numeric(as.character(metadata$Longitude)) * 12 / 180,
          keep_na = FALSE)

write_sef(out[,c("year","month","day","hour","minute","ta")],
          outpath = paste0("../../4_Formatted/", name_folder),
          variable = "ta",
          cod = metadata$Code,
          nam = metadata$Station,
          lat = metadata$Latitude,
          lon = metadata$Longitude,
          alt = metadata$Altitude,
          sou = "PALAEO-RA",
          link = metadata$C3S.Link,
          units = "C",
          stat = "point",
          metaHead = paste0("Observer=", metadata$Observer, "|Instrument=Palatine Society"),
          meta = out$ta_meta,
          period = 0,
          time_offset = as.numeric(as.character(metadata$Longitude)) * 12 / 180,
          keep_na = FALSE)

write_sef(out[,c("year","month","day","hour","minute","dd")],
          outpath = paste0("../../4_Formatted/", name_folder),
          variable = "dd",
          cod = metadata$Code,
          nam = metadata$Station,
          lat = metadata$Latitude,
          lon = metadata$Longitude,
          alt = metadata$Altitude,
          sou = "PALAEO-RA",
          link = metadata$C3S.Link,
          units = "degree",
          stat = "point",
          metaHead = paste0("Observer=", metadata$Observer),
          meta = out$dd_meta,
          period = 0,
          time_offset = as.numeric(as.character(metadata$Longitude)) * 12 / 180,
          keep_na = TRUE)

write_sef(out[,c("year","month","day","hour","minute","w")],
          outpath = paste0("../../4_Formatted/", name_folder),
          variable = "w",
          cod = metadata$Code,
          nam = metadata$Station,
          lat = metadata$Latitude,
          lon = metadata$Longitude,
          alt = metadata$Altitude,
          sou = "PALAEO-RA",
          link = metadata$C3S.Link,
          units = "0-4",
          stat = "point",
          metaHead = paste0("Observer=", metadata$Observer),
          meta = out$w_meta,
          period = 0,
          time_offset = as.numeric(as.character(metadata$Longitude)) * 12 / 180,
          keep_na = FALSE)



### DAILY

#read metadata
metadata <- inventory[which(inventory$Code == station_code & 
                              inventory$Resolution == "daily" &
                              inventory$First.Year == "1781"), ]
#files
files <- "Palatine-Society_T3_CZ_Prague_1787_daily.xlsx"

#read data
template <- read.xlsx(paste0("../../3_Revised/Prague/",files),
                      sheetIndex=1, startRow=5, header=FALSE)

#time
hour <- rep(NA, 365)
minute <- rep(NA, 365)
dates <- as.Date(paste(template[,1], template[,2], template[,3], sep="-"))
meta_time <- paste0("orig.date=", dates)

#initialize data frame
out <- data.frame(year = as.integer(template[,1]), 
                  month = as.integer(template[,2]), 
                  day = as.integer(template[,3]),
                  hour = hour, minute = minute)

#pressure
template[,4] <- na.locf(template[,4])
p_in <- as.numeric(template[,4]) + as.numeric(template[,5]) / 12
template$atb <- NA
for (k in c(1:12)) {
  template$atb[which(template[,2] == k)] <- round(lm[[k]]$coefficients[1] +
        template$X6[which(template[,2] == k)] * lm[[k]]$coefficients[2], 1)}
out$p <- round(convert_pressure(p_in, 27.07, 
                                as.numeric(metadata$Latitude), 
                                as.numeric(metadata$Altitude),
                                template$atb*1.25), 1)
out$p_meta <- paste0(meta_time, "|orig=", template[,4], ".", template[,5],
                     "Pin", "|atb=", template$atb, "R (extrapolated from ta)")

#temperature
out$ta <- round(as.numeric(as.character(template[,6])) * 1.25, 1)
out$ta_meta <- paste0(meta_time, "|orig=", template[,6], "R")


#write SEF files
write_sef(out[,c("year","month","day","hour","minute","p")],
          outpath = paste0("../../4_Formatted/", name_folder),
          variable = "p",
          cod = metadata$Code,
          nam = metadata$Station,
          lat = metadata$Latitude,
          lon = metadata$Longitude,
          alt = metadata$Altitude,
          sou = "PALAEO-RA",
          link = metadata$C3S.Link,
          units = "hPa",
          stat = "mean",
          metaHead = paste0("Observer=", metadata$Observer, "|PGC=Y|PTC=Y|Instrument=Palatine Society"),
          meta = out$p_meta,
          period = "day",
          time_offset = as.numeric(as.character(metadata$Longitude)) * 12 / 180,
          keep_na = FALSE,
          note = "daily")

write_sef(out[,c("year","month","day","hour","minute","ta")],
          outpath = paste0("../../4_Formatted/", name_folder),
          variable = "ta",
          cod = metadata$Code,
          nam = metadata$Station,
          lat = metadata$Latitude,
          lon = metadata$Longitude,
          alt = metadata$Altitude,
          sou = "PALAEO-RA",
          link = metadata$C3S.Link,
          units = "C",
          stat = "mean",
          metaHead = paste0("Observer=", metadata$Observer),
          meta = out$ta_meta,
          period = "day",
          time_offset = as.numeric(as.character(metadata$Longitude)) * 12 / 180,
          keep_na = TRUE,
          note = "daily")


#precipitation
#files
files <- "Palatine-Society_T3_CZ_Prague_1781-1786_daily-prec.xlsx"

#read data
template <- read.xlsx(paste0("../../3_Revised/Prague/",files),
                      sheetIndex=1, startRow=5, header=FALSE)

#time
hour <- rep(NA, 1979)
minute <- rep(NA, 1979)
dates <- as.Date(paste(template[,2], template[,3], template[,4], sep="-"))
meta_time <- paste0("orig.date=", dates)

#initialize data frame
out <- data.frame(year = as.integer(template[,2]), 
                  month = as.integer(template[,3]), 
                  day = as.integer(template[,4]),
                  hour = hour, minute = minute)

# some rainfall observations mistakenly digitised as 'whole numbers' rather than decimals 
template$X5 <- as.character(template$X5)
int <- which(!grepl("\\.",template[,5]))
template <- template %>%
  separate(5, into = c("lin", "grana"))
template[int,6] <- template[int,5]
template[int,5] <- 0
out$rr <- round((as.numeric(template$lin) + as.numeric(template$grana) /100) /12 * 27.07, 1)
out$rr[c(1250:1734,1827:1979)] <- NA
out$rr_meta <- paste0(meta_time, "|orig=", as.numeric(template$lin) + as.numeric(template$grana), "lin.grana")
#plot(out$rr)
write_sef(out[,c("year","month","day","hour","minute","rr")],
          outpath = paste0("../../4_Formatted/", name_folder),
          variable = "rr",
          cod = metadata$Code,
          nam = metadata$Station,
          lat = metadata$Latitude,
          lon = metadata$Longitude,
          alt = metadata$Altitude,
          sou = "PALAEO-RA",
          link = metadata$C3S.Link,
          units = "mm",
          stat = "sum",
          metaHead = paste0("Observer=", metadata$Observer),
          meta = out$rr_meta,
          period = "day",
          time_offset = as.numeric(as.character(metadata$Longitude)) * 12 / 180,
          keep_na = TRUE,
          note = "daily")



### MONTHLY

#read metadata
metadata <- inventory[which(inventory$Code == station_code & 
                              inventory$Resolution == "monthly" &
                              inventory$First.Year == "1789"), ]
#files
files <- c("Palatine-Society_T3_CZ_Prague_1789_monthly.xlsx")

#read data
template <- read.xlsx(paste0("../../3_Revised/Prague/",files),
                      sheetIndex=1, startRow=6, header=FALSE)

#initialize data frame
out <- data.frame(year = as.integer(template[,1]), 
                  month = as.integer(template[,2]), 
                  day = rep(NA, nrow(template)), 
                  hour = rep(NA, nrow(template)),
                  minute = rep(NA, nrow(template)))

#pressure
p_in_max <- as.numeric(template[,3]) + as.numeric(template[,4]) / 12
out$pmax <- round(convert_pressure(p_in_max, 27.07,
                                   as.numeric(metadata$Latitude), 
                                   as.numeric(as.character(metadata$Altitude))), 1)
#out$day <- template[,5]
out$pmax_meta <- paste0("orig=", template[,3], ".", template[,4], "Pin")
write_sef(out[,c("year","month","day","hour","minute","pmax")],
          outpath = paste0("../../4_Formatted/", name_folder),
          variable = "p",
          cod = metadata$Code,
          nam = metadata$Station,
          lat = metadata$Latitude,
          lon = metadata$Longitude,
          alt = metadata$Altitude,
          sou = "PALAEO-RA",
          link = metadata$C3S.Link,
          units = "hPa",
          stat = "maximum",
          metaHead = paste0("Observer=", metadata$Observer, 
                            "|PGC=Y|PTC=N|Instrument=Palatine Society"),
          meta = out$pmax_meta,
          period = "month",
          time_offset = as.numeric(as.character(metadata$Longitude)) * 12 / 180,
          keep_na = FALSE,
          note = "monthly-max")
p_in_min <- as.numeric(template[,6]) + as.numeric(template[,7]) / 12
out$pmin <- round(convert_pressure(p_in_min, 27.07,
                                   as.numeric(metadata$Latitude), 
                                   as.numeric(as.character(metadata$Altitude))), 1)
#out$day <- template[,8]
out$pmin_meta <- paste0("orig=", template[,6], ".", template[,7], "Pin")
write_sef(out[,c("year","month","day","hour","minute","pmin")],
          outpath = paste0("../../4_Formatted/", name_folder),
          variable = "p",
          cod = metadata$Code,
          nam = metadata$Station,
          lat = metadata$Latitude,
          lon = metadata$Longitude,
          alt = metadata$Altitude,
          sou = "PALAEO-RA",
          link = metadata$C3S.Link,
          units = "hPa",
          stat = "minimum",
          metaHead = paste0("Observer=", metadata$Observer, 
                            "|PGC=Y|PTC=N|Instrument=Palatine Society"),
          meta = out$pmin_meta,
          period = "month",
          time_offset = as.numeric(as.character(metadata$Longitude)) * 12 / 180,
          keep_na = FALSE,
          note = "monthly-min")
template$atb <- NA
for (k in c(1:12)) {
  template$atb[which(template[,2] == k)] <- round(lm[[k]]$coefficients[1] +
       template$X15[which(template[,2] == k)] * lm[[k]]$coefficients[2], 1)}
p_in_med <- as.numeric(template[,9]) + as.numeric(template[,10]) / 12
out$pmed <- round(convert_pressure(p_in_med, 27.07,
                                   as.numeric(metadata$Latitude), 
                                   as.numeric(as.character(metadata$Altitude)),
                                   template$atb*1.25), 1)
#out$day <- rep(NA, 12)
out$pmed_meta <- paste0("orig=", template[,9], ".", template[,10], "Pin", "|atb=", template$atb, "R (extrapolated from ta)")
write_sef(out[,c("year","month","day","hour","minute","pmed")],
          outpath = paste0("../../4_Formatted/", name_folder),
          variable = "p",
          cod = metadata$Code,
          nam = metadata$Station,
          lat = metadata$Latitude,
          lon = metadata$Longitude,
          alt = metadata$Altitude,
          sou = "PALAEO-RA",
          link = metadata$C3S.Link,
          units = "hPa",
          stat = "mid_range",
          metaHead = paste0("Observer=", metadata$Observer, 
                            "|PGC=Y|PTC=Y|Instrument=Palatine Society"),
          meta = out$pmed_meta,
          period = "month",
          time_offset = as.numeric(as.character(metadata$Longitude)) * 12 / 180,
          keep_na = FALSE,
          note = "monthly-midrange")

#temperature
out$tmax <- round(template[,11]*1.25, 1)
#out$day <- template[,12]
out$tmax_meta <- paste0("orig=", template[,11], "R")
write_sef(out[,c("year","month","day","hour","minute","tmax")],
          outpath = paste0("../../4_Formatted/", name_folder),
          variable = "ta",
          cod = metadata$Code,
          nam = metadata$Station,
          lat = metadata$Latitude,
          lon = metadata$Longitude,
          alt = metadata$Altitude,
          sou = "PALAEO-RA",
          link = metadata$C3S.Link,
          units = "C",
          stat = "maximum",
          metaHead = paste0("Observer=", metadata$Observer, " |Instrument=Palatine Society"),
          meta = out$tmax_meta,
          period = "month",
          time_offset = as.numeric(as.character(metadata$Longitude)) * 12 / 180,
          keep_na = FALSE,
          note = "monthly-max")
out$tmin <- round(template[,13]*1.25, 1)
#out$day <- template[,14]
out$tmin_meta <- paste0("orig=", template[,13], "R")
write_sef(out[,c("year","month","day","hour","minute","tmin")],
          outpath = paste0("../../4_Formatted/", name_folder),
          variable = "ta",
          cod = metadata$Code,
          nam = metadata$Station,
          lat = metadata$Latitude,
          lon = metadata$Longitude,
          alt = metadata$Altitude,
          sou = "PALAEO-RA",
          link = metadata$C3S.Link,
          units = "C",
          stat = "minimum",
          metaHead = paste0("Observer=", metadata$Observer, " |Instrument=Palatine Society"),
          meta = out$tmin_meta,
          period = "month",
          time_offset = as.numeric(as.character(metadata$Longitude)) * 12 / 180,
          keep_na = FALSE,
          note = "monthly-min")
out$tmed <- round(template[,15]*1.25, 1)
#out$day <- rep(NA, 12)
out$tmed_meta <- paste0("orig=", template[,15], "R")
write_sef(out[,c("year","month","day","hour","minute","tmed")],
          outpath = paste0("../../4_Formatted/", name_folder),
          variable = "ta",
          cod = metadata$Code,
          nam = metadata$Station,
          lat = metadata$Latitude,
          lon = metadata$Longitude,
          alt = metadata$Altitude,
          sou = "PALAEO-RA",
          link = metadata$C3S.Link,
          units = "C",
          stat = "mid_range",
          metaHead = paste0("Observer=", metadata$Observer, " |Instrument=Palatine Society"),
          meta = out$tmed_meta,
          period = "month",
          time_offset = as.numeric(as.character(metadata$Longitude)) * 12 / 180,
          keep_na = FALSE,
          note = "monthly-midrange")



#check SEF files
for (f in list.files(paste0("../../4_Formatted/", name_folder), full.names=TRUE)) {
  print(f)
  check_sef(f)
}
