library(dataresqc)
library(tidyr)
library(tibble)
library(zoo)
library(xlsx)
library(reshape2)
library(lubridate)

##################################
###          STOCKHOLM
##################################


#create station code
name_folder <- strsplit(getwd(), "/")[[1]][8]
name_group <- strsplit(getwd(), "/")[[1]][6]
station_code <- paste(name_group, name_folder, sep = "_")

#read inventory
inventory <- read.xlsx("/scratch3/PALAEO-RA/DataRescue/Inventory_PALAEO-RA_digi.xls", 
                       sheetIndex=1, colTypes="character")


#read metadata
metadata <- inventory[which(inventory$Code == station_code & 
                              inventory$Resolution == "subdaily"), ]
#metadata
#lat <- 59.21
lat <- as.numeric(metadata$Latitude)
#lon <- 18.03
lon <- as.numeric(metadata$Longitude)
#alt <- 44.4
alt <- as.numeric(metadata$Altitude)


### SUBDAILY

#files
files <- list.files("../../3_Revised/Stockholm", pattern="subdaily")

#extrapolation of atb from ta with lm
template <- list()
for (i in c(1:2)) {
  template[[i]] <- suppressWarnings(read.xlsx(paste0("../../3_Revised/Stockholm/",files[i]),
                             sheetIndex=1, startRow=3, header=FALSE)[,1:9])
}
template <- Reduce(rbind, template)
xmean <- round(aggregate(template[,c(8:9)], list(template[,1],template[,2],template[,3]), mean, na.rm=TRUE), 1)
xmean <- xmean[order(xmean$Group.1, xmean$Group.2, xmean$Group.3), ]
lm <- list()
for (i in c(1:12)) {
  j <- which(xmean[,2] == i)
  lm[[i]] <- lm(xmean[j,4]~xmean[j,5])
}

##loop on files
out <- list()
for (i in 1:length(files)) {
  
  #read data
  template <- read.xlsx(paste0("../../3_Revised/Stockholm/",files[i]),
                        sheetIndex=1, startRow=3, header=FALSE)

  #time
  time <- paste0(template[,4], ":", sprintf("%02d", template[,5]))
  hour <- as.integer(template[,4])
  minute <- as.integer(template[,5])
  date <- as.Date(paste(template[,1], sprintf("%02d", template[,2]), 
                        sprintf("%02d", template[,3]), sep="-"))
  meta_time <- paste0("orig.date=", date, "|orig.time=", time)
  
  #initialize data frame
  out[[i]] <- data.frame(year = as.integer(template[,1]), 
                         month = as.integer(template[,2]), 
                         day = as.integer(template[,3]), 
                         hour = hour, minute = minute)
  
  #pressure
  if (grepl("83", files[i])) {
    template$atb <- NA
    for (k in c(1:12)) {
      template$atb[which(template[,2] == k)] <- round(lm[[k]]$coefficients[1] +
          template[,9][which(template[,2] == k)] * lm[[k]]$coefficients[2], 1)}
    template$X8[which(is.na(template$X8))] <- template$atb[which(is.na(template$X8))]
    template[,6] <- na.locf(template[,6])
    p_in <- as.numeric(template[,6]) + as.numeric(template[,7]) / 12
    out[[i]]$p <- round(convert_pressure(p_in, 27.07, 
                                         lat, alt,
                                         as.numeric(template[,8])*1.25), 1)
    i_miss <- which(is.na(as.numeric(template[,8])))
    out[[i]]$p[i_miss] <- round(convert_pressure(p_in[i_miss], 27.07, 
                                                 lat, alt), 1)
    out[[i]]$p_meta <- paste0(meta_time, "|orig=", template[,6], ".", template[,7],
                              "Pin|atb=", template[,8], "R")
    out[[1]]$p_meta[1:1826] <- paste0(out[[1]]$p_meta[1:1826], " (extrapolated from ta)")
    out[[i]]$p_meta[i_miss] <- paste0(meta_time[i_miss], "|orig=", template[i_miss,6], ".", 
                                      template[i_miss,7], "Pin|atb=", template[i_miss,8], "|PTC=N")
  } else if (grepl("88", files[i])) {
    template[,6] <- na.locf(template[,6])
    p_in <- as.numeric(template[,6]) + as.numeric(template[,7]) / 12
    out[[i]]$p <- round(convert_pressure(p_in, 27.07, 
                                         lat, alt,
                                         template[,8]*1.25), 1)
    out[[i]]$p_meta <- paste0(meta_time, "|orig=", template[,6], ".", template[,7],
                              "Pin|atb=", template$X8, "R")
  } else {
    out[[i]]$p <- NA
    out[[i]]$p_meta <- ""
  }
  
  #temperature
  out[[i]]$ta <- round(as.numeric(as.character(template$X9)) * 1.25, 1)
  out[[i]]$ta_meta <- paste0(meta_time, "|orig=", template$X9, "R")

}


#merge dataframes
out <- Reduce(rbind, out)

#write SEF files
write_sef(out[,c("year","month","day","hour","minute","p")],
          outpath = paste0("../../4_Formatted/", name_folder),
          variable = "p",
          cod = metadata$Code,
          nam = metadata$Station,
          lat = metadata$Latitude,
          lon = metadata$Longitude,
          alt = metadata$Altitude,
          sou = "PALAEO-RA",
          link = metadata$C3S.Link,
          units = "hPa",
          stat = "point",
          metaHead = paste0("Observer=",metadata$Observer,"|PGC=Y|PTC=Y|Instrument=Palatine Society"),
          meta = out$p_meta,
          period = 0,
          time_offset = as.numeric(as.character(lon)) * 12 / 180,
          keep_na = FALSE)

write_sef(out[,c("year","month","day","hour","minute","ta")],
          outpath = paste0("../../4_Formatted/", name_folder),
          variable = "ta",
          cod = metadata$Code,
          nam = metadata$Station,
          lat = metadata$Latitude,
          lon = metadata$Longitude,
          alt = metadata$Altitude,
          sou = "PALAEO-RA",
          link = metadata$C3S.Link,
          units = "C",
          stat = "point",
          metaHead = paste0("Observer=",metadata$Observer),
          meta = out$ta_meta,
          period = 0,
          time_offset = as.numeric(as.character(lon)) * 12 / 180,
          keep_na = FALSE)



### MONTHLY

#files
files <- list.files("../../3_Revised/Stockholm", pattern="monthly")

#read data
template <- read.xlsx(paste0("../../3_Revised/Stockholm/",files),
                      sheetIndex=1, startRow=4, header=FALSE)

#initialize data frame
out <- data.frame(year = as.integer(template[,1]), 
                  month = as.integer(template[,2]), 
                  day = rep(NA, nrow(template)), 
                  hour = rep(NA, nrow(template)),
                  minute = rep(NA, nrow(template)))

template$atb <- NA
#pressure
p_in_max <- as.numeric(template[,3]) + as.numeric(template[,4]) / 12
out$pmax <- round(convert_pressure(p_in_max, 27.07,
                                   lat, alt), 1)
out$pmax_meta <- paste0("orig=", template[,3], ".", template[,4], "Pin")
write_sef(out[,c("year","month","day","hour","minute","pmax")],
          outpath = paste0("../../4_Formatted/", name_folder),
          variable = "p",
          cod = metadata$Code,
          nam = metadata$Station,
          lat = metadata$Latitude,
          lon = metadata$Longitude,
          alt = metadata$Altitude,
          sou = "PALAEO-RA",
          link = metadata$C3S.Link,
          units = "hPa",
          stat = "maximum",
          metaHead = paste0("Observer=",metadata$Observer,"|PGC=Y|PTC=N|Instrument=Palatine Society"),
          meta = out$pmax_meta,
          period = "month",
          time_offset = as.numeric(as.character(lon)) * 12 / 180,
          keep_na = FALSE,
          note = "monthly-max")
p_in_min <- as.numeric(template[,6]) + as.numeric(template[,7]) / 12
out$pmin <- round(convert_pressure(p_in_min, 27.07,
                                   lat, alt), 1)
out$pmin_meta <- paste0("orig=", template[,6], ".", template[,7], "Pin")
write_sef(out[,c("year","month","day","hour","minute","pmin")],
          outpath = paste0("../../4_Formatted/", name_folder),
          variable = "p",
          cod = metadata$Code,
          nam = metadata$Station,
          lat = metadata$Latitude,
          lon = metadata$Longitude,
          alt = metadata$Altitude,
          sou = "PALAEO-RA",
          link = metadata$C3S.Link,
          units = "hPa",
          stat = "minimum",
          metaHead = paste0("Observer=",metadata$Observer,"|PGC=Y|PTC=N|Instrument=Palatine Society"),
          meta = out$pmin_meta,
          period = "month",
          time_offset = as.numeric(as.character(lon)) * 12 / 180,
          keep_na = FALSE,
          note = "monthly-min")
for (k in c(1:12)) {
  template$atb[which(template[,2] == k)] <- round(lm[[k]]$coefficients[1] +
      template$X15[which(template[,2] == k)] * lm[[k]]$coefficients[2], 1)}
p_in_med <- as.numeric(template[,9]) + as.numeric(template[,10]) / 12
out$pmed <- round(convert_pressure(p_in_med, 27.07,
                                   lat, alt,
                                   template$atb*1.25), 1)
out$pmed_meta <- paste0("orig=", template[,9], ".", template[,10], "Pin", "|atb=", template$atb, "R (extrapolated from ta)")
write_sef(out[,c("year","month","day","hour","minute","pmed")],
          outpath = paste0("../../4_Formatted/", name_folder),
          variable = "p",
          cod = metadata$Code,
          nam = metadata$Station,
          lat = metadata$Latitude,
          lon = metadata$Longitude,
          alt = metadata$Altitude,
          sou = "PALAEO-RA",
          link = metadata$C3S.Link,
          units = "hPa",
          stat = "mean",
          metaHead = paste0("Observer=",metadata$Observer,"|PGC=Y|PTC=N|Instrument=Palatine Society"),
          meta = out$pmed_meta,
          period = "month",
          time_offset = as.numeric(as.character(lon)) * 12 / 180,
          keep_na = FALSE,
          note = "monthly-med")

#temperature
out$tmax <- round(template[,11]*1.25, 1)
out$tmax_meta <- paste0("orig=", template[,11], "R")
write_sef(out[,c("year","month","day","hour","minute","tmax")],
          outpath = paste0("../../4_Formatted/", name_folder),
          variable = "ta",
          cod = metadata$Code,
          nam = metadata$Station,
          lat = metadata$Latitude,
          lon = metadata$Longitude,
          alt = metadata$Altitude,
          sou = "PALAEO-RA",
          link = metadata$C3S.Link,
          units = "C",
          stat = "maximum",
          metaHead = paste0("Observer=",metadata$Observer),
          meta = out$tmax_meta,
          period = "month",
          time_offset = as.numeric(as.character(lon)) * 12 / 180,
          keep_na = FALSE,
          note = "monthly-max")
out$tmin <- round(template[,13]*1.25, 1)
out$tmin_meta <- paste0("orig=", template[,13], "R")
write_sef(out[,c("year","month","day","hour","minute","tmin")],
          outpath = paste0("../../4_Formatted/", name_folder),
          variable = "ta",
          cod = metadata$Code,
          nam = metadata$Station,
          lat = metadata$Latitude,
          lon = metadata$Longitude,
          alt = metadata$Altitude,
          sou = "PALAEO-RA",
          link = metadata$C3S.Link,
          units = "C",
          stat = "minimum",
          metaHead = paste0("Observer=",metadata$Observer),
          meta = out$tmin_meta,
          period = "month",
          time_offset = as.numeric(as.character(lon)) * 12 / 180,
          keep_na = FALSE,
          note = "monthly-min")
out$tmed <- round(template[,15]*1.25, 1)
out$tmed_meta <- paste0("orig=", template[,15], "R")
write_sef(out[,c("year","month","day","hour","minute","tmed")],
          outpath = paste0("../../4_Formatted/", name_folder),
          variable = "ta",
          cod = metadata$Code,
          nam = metadata$Station,
          lat = metadata$Latitude,
          lon = metadata$Longitude,
          alt = metadata$Altitude,
          sou = "PALAEO-RA",
          link = metadata$C3S.Link,
          units = "C",
          stat = "mean",
          metaHead = paste0("Observer=",metadata$Observer),
          meta = out$tmed_meta,
          period = "month",
          time_offset = as.numeric(as.character(lon)) * 12 / 180,
          keep_na = FALSE,
          note = "monthly-med")



#check SEF files
for (f in list.files(paste0("../../4_Formatted/", name_folder), full.names=TRUE)) {
#  print(f)
  check_sef(f)
}
