library(dataresqc)
library(tidyr)
library(zoo)
library(readxl)

##################################
###          ZAGAN
##################################

#create station code
name_folder <- "Zagan"
name_group <- "Palatine-Society"
station_code <- paste(name_group, name_folder, sep = "_")

#define wind directions
directions <- c("N", "NbE", "NNE", "NEbN", "NE", "NEbE", "ENE", "EbN", "E", 
                "EbS", "ESE", "SEbE", "SE", "SEbS", "SSE", "SbE", "S",
                "SbW", "SSW", "SWbS", "SW", "SWbW", "WSW", "WbS", "W",
                "WbN", "WNW", "NWbW", "NW", "NWbN", "NNW", "NbW")


### SUBDAILY

#metadata
lat <- 51.37
lon <- 15.19
alt <- 116

#files
files <- c("Zagan_subdaily_1781.xlsx", "Zagan_subdaily_1782.xlsx",
           "Zagan_subdaily_1783.xlsx", "Zagan_subdaily_1784.xlsx",
           "Zagan_subdaily_1785.xlsx", "Zagan_subdaily_1786.xlsx",
           "Zagan_subdaily_1787.xlsx", "Zagan_subdaily_1788.xlsx",
           "Zagan_subdaily_1789.xlsx", "Zagan_subdaily_1790.xlsx",
           "Zagan_subdaily_1791.xlsx", "Zagan_subdaily_1792.xlsx")

#extrapolation of atb from ta with lm
template <- list()
for (i in c(1:6,9:12)) {
  template[[i]] <- data.frame(read_xlsx(paste0("./", files[i]), 
                                        sheet=1, col_names=T)[,1:9])
}
template <- Reduce(rbind, template)
xmean <- round(aggregate(template[,8:9], list(template[,1],template[,2],template[,3]), mean, na.rm=TRUE), 1)
xmean <- xmean[order(xmean$Group.1, xmean$Group.2, xmean$Group.3), ]
xmean <- xmean[-c(1:365),] # 1781 atb is incomplete, will not be part of lm
lm <- list()
for (i in c(1:12)) {
  j <- which(xmean[,2] == i)
  lm[[i]] <- lm(xmean[j,4]~xmean[j,5])
}

##loop on files
out <- list()
for (i in 1:length(files)) {
  
  #read data
  template <- data.frame(read_xlsx(paste0("./", files[i]), sheet=1, col_names=T))
  
  #time
  time <- paste0(template[,4], ":", sprintf("%02d", template[,5]))
  hour <- as.integer(template[,4])
  minute <- as.integer(template[,5])
  date <- as.Date(paste(template[,1], sprintf("%02d", template[,2]), 
                        sprintf("%02d", template[,3]), sep="-"))
  meta_time <- paste0("orig.date=", date, "|orig.time=", time)
  
  #initialize data frame
  out[[i]] <- data.frame(year = as.integer(template[,1]), 
                         month = as.integer(template[,2]), 
                         day = as.integer(template[,3]), 
                         hour = hour, minute = minute)
  
  #pressure
  if (!grepl("87", files[i])) {
    template[,8] <- template[,8] %>% replace_na(13.73) #room temp climatology in ?R for Zagan
    atb <- round((template[,8] * 1.25), 2) 
    template[,6] <- na.locf(template[,6])
    p_in <- as.numeric(template[,6]) + as.numeric(template[,7]) / 12
    out[[i]]$p <- round(convert_pressure(p_in, 27.07, lat, alt, atb), 1)
    out[[i]]$p_meta <- paste0(meta_time, "|orig=", template[,6], ".", template[,7],
                              "Pin|atb=", template[,8], "R")
  } else {
    out[[i]]$p <- NA
    out[[i]]$p_meta <- ""
  }
  
  #temperature
  if (!grepl("87", files[i])) {
    out[[i]]$ta <- round(template[,9] * 1.25, 1)
    out[[i]]$ta_meta <- paste0(meta_time, "|orig=", template[,9], "R")
  } else {
    out[[i]]$ta <- NA
    out[[i]]$ta_meta <- ""
  }

  #wind direction
  out[[i]]$dd_meta <- paste0(meta_time, "|orig=", template$dd)
  directions <- gsub("E", "O", directions)
  template$dd[grepl("&",template$dd)] <- sub(".* ", "", template$dd[grepl("&",template$dd)])
  out[[i]]$dd <- round(11.25 * (match(as.character(template$dd), directions) - 1), 0)

  #wind force
  out[[i]]$w_meta <- paste0(meta_time, "|orig=", as.character(template$w))
  template$w[grepl(" ",template$w)] <- sub(".*\\. |.* & | O.*", "", template$w[grepl(" ",template$w)])
  template$w[grepl("\\.|\\+|O",template$w)] <- sub("\\.| .*| O.*", "", template$w[grepl("\\.|\\+|O",template$w)])
  template$w <- gsub("Orcan", "4", template$w)
  out[[i]]$w <- as.integer(as.character(template$w))

  #precipitation
  if (!grepl("87", files[i])) {
    template$precip. <- as.numeric(as.character(template$precip.))
    template$precip.[is.na(template$precip.)] <- 0
    out[[i]]$rr <- round((template$precip. /64 /12) * 27.07, 1)
    out[[i]]$rr_meta <- paste0(meta_time, "|orig=", template$precip., "[lin./64]")
  } else {
    out[[i]]$rr <- NA
    out[[i]]$rr_meta <- ""
  }

  #fresh snow
  if (!grepl("81|87", files[i])) {
    template$precip.[is.na(template$precip.)] <- 0
    template$precip.[!grepl("n",template$rr.specif.)] <- 0
    out[[i]]$fs <- round((template$precip. /64 /12) * 27.07, 1)
    out[[i]]$fs_meta <- paste0(meta_time, "|orig=", template$precip., "[lin./64]")
  } else {
    out[[i]]$fs <- NA
    out[[i]]$fs_meta <- ""
  }
}


#minor adjustments
out[[1]]$w[which(out[[1]]$w==33)] <- NA
out[[12]]$w[which(out[[12]]$w==0)] <- NA

#merge dataframes
out <- Reduce(rbind, out)

#write SEF files
write_sef(out[,c("year","month","day","hour","minute","p")],
          outpath = "SEF",
          variable = "p",
          cod = station_code,
          nam = "Zagan",
          lat = lat,
          lon = lon,
          alt = alt,
          sou = "PALAEO-RA",
          link = "NA",
          units = "hPa",
          stat = "point",
          metaHead = paste0("Observer=", "Preuss", 
                            "|PGC=Y|PTC=Y|Instrument=Palatine Society"),
          meta = out$p_meta,
          period = 0,
          time_offset = as.numeric(as.character(lon)) * 12 / 180,
          keep_na = FALSE)

write_sef(out[,c("year","month","day","hour","minute","ta")],
          outpath = "SEF",
          variable = "ta",
          cod = station_code,
          nam = "Zagan",
          lat = lat,
          lon = lon,
          alt = alt,
          sou = "PALAEO-RA",
          link = "NA",
          units = "C",
          stat = "point",
          metaHead = paste0("Observer=", "Preuss", "|Instrument=Palatine Society"),
          meta = out$ta_meta,
          period = 0,
          time_offset = as.numeric(as.character(lon)) * 12 / 180,
          keep_na = FALSE)

write_sef(out[,c("year","month","day","hour","minute","dd")],
          outpath = "SEF",
          variable = "dd",
          cod = station_code,
          nam = "Zagan",
          lat = lat,
          lon = lon,
          alt = alt,
          sou = "PALAEO-RA",
          link = "NA",
          units = "degree",
          stat = "point",
          metaHead = paste0("Observer=", "Preuss"),
          meta = out$dd_meta,
          period = 0,
          time_offset = as.numeric(as.character(lon)) * 12 / 180,
          keep_na = TRUE)

write_sef(out[,c("year","month","day","hour","minute","w")],
          outpath = "SEF",
          variable = "w",
          cod = station_code,
          nam = "Zagan",
          lat = lat,
          lon = lon,
          alt = alt,
          sou = "PALAEO-RA",
          link = "NA",
          units = "0-4",
          stat = "point",
          metaHead = paste0("Observer=", "Preuss"),
          meta = out$w_meta,
          period = 0,
          time_offset = as.numeric(as.character(lon)) * 12 / 180,
          keep_na = FALSE)

write_sef(out[,c("year","month","day","hour","minute","rr")],
          outpath = "SEF",
          variable = "rr",
          cod = station_code,
          nam = "Zagan",
          lat = lat,
          lon = lon,
          alt = alt,
          sou = "PALAEO-RA",
          link = "NA",
          units = "mm",
          stat = "sum",
          metaHead = paste0("Observer=", "Preuss"),
          meta = out$rr_meta,
          period = "p",
          time_offset = as.numeric(as.character(lon)) * 12 / 180,
          keep_na = FALSE)

write_sef(out[,c("year","month","day","hour","minute","fs")],
          outpath = "SEF",
          variable = "fs",
          cod = station_code,
          nam = "Zagan",
          lat = lat,
          lon = lon,
          alt = alt,
          sou = "PALAEO-RA",
          link = "NA",
          units = "mm",
          stat = "sum",
          metaHead = paste0("Observer=", "Preuss"),
          meta = out$fs_meta,
          period = "p",
          time_offset = as.numeric(as.character(lon)) * 12 / 180,
          keep_na = FALSE)


# MONTHLY

#read data
template <- data.frame(read_xlsx("./Zagan_monthly_1787.xlsx", sheet=1, skip=4, col_names=T))

#initialize data frame
out <- data.frame(year = as.integer(template[,1]), month = as.integer(template[,2]), 
                  day = rep(NA, 12), hour = rep(NA, 12), minute = rep(NA, 12))

#pressure
#max
p_in <- as.numeric(template[,3]) + as.numeric(template[,4]) / 12
out$p_max <- round(convert_pressure(p_in, 27.07, lat, alt), 1)
out$p_meta_max <- paste0("orig=", template[,3], ".", template[,4], "Pin")
out$day_max <- template[,5]
#min
p_in <- as.numeric(template[,6]) + as.numeric(template[,7]) / 12
out$p_min <- round(convert_pressure(p_in, 27.07, lat, alt), 1)
out$p_meta_min <- paste0("orig=", template[,6], ".", template[,7], "Pin")
out$day_min <- template[,8]
#med
template$atb <- NA
for (k in c(1:12)) {
  template$atb[which(template[,2] == k)] <- round(lm[[k]]$coefficients[1] +
        template[,15][which(template[,2] == k)] * lm[[k]]$coefficients[2], 1)}
p_in <- as.numeric(template[,9]) + as.numeric(template[,10]) / 12
out$p_med <- round(convert_pressure(p_in, 27.07, lat, alt, template$atb), 1)
out$p_meta_med <- paste0("orig=", template[,9], ".", template[,10], "Pin|atb=", template$atb, "R (extrapolated from ta)")

#write SEF files
write_sef(out[,c("year","month","day","hour","minute","p_max")],
          outpath = "SEF",
          variable = "p", cod = station_code,
          nam = "Zagan", lat = lat,
          lon = lon, alt = alt,
          sou = "PALAEO-RA", link = "NA",
          units = "hPa", stat = "maximum",
          metaHead = paste0("Observer=", "Preuss", 
                            "|PGC=Y|PTC=N|Instrument=Palatine Society"),
          meta = out$p_meta_max, period = "month",
          time_offset = as.numeric(as.character(lon)) * 12 / 180,
          keep_na = FALSE, note = "monthly-max")
write_sef(out[,c("year","month","day","hour","minute","p_min")],
          outpath = "SEF",
          variable = "p", cod = station_code,
          nam = "Zagan", lat = lat,
          lon = lon, alt = alt,
          sou = "PALAEO-RA", link = "NA",
          units = "hPa", stat = "minimum",
          metaHead = paste0("Observer=", "Preuss", 
                            "|PGC=Y|PTC=N|Instrument=Palatine Society"),
          meta = out$p_meta_min, period = "month",
          time_offset = as.numeric(as.character(lon)) * 12 / 180,
          keep_na = FALSE, note = "monthly-min")
write_sef(out[,c("year","month","day","hour","minute","p_med")],
          outpath = "SEF",
          variable = "p", cod = station_code,
          nam = "Zagan", lat = lat,
          lon = lon, alt = alt,
          sou = "PALAEO-RA", link = "NA",
          units = "hPa", stat = "mid_range",
          metaHead = paste0("Observer=", "Preuss", 
                            "|PGC=Y|PTC=Y|Instrument=Palatine Society"),
          meta = out$p_meta_med, period = "month",
          time_offset = as.numeric(as.character(lon)) * 12 / 180,
          keep_na = FALSE, note = "monthly-midrange")

#temperature
#max
out$ta_max <- round(template[,11] * 1.25, 1)
out$ta_meta_max <- paste0("orig=", template[,11], "R")
out$day_max <- template[,12]
#min
out$ta_min <- round(template[,13] * 1.25, 1)
out$ta_meta_min <- paste0("orig=", template[,13], "R")
out$day_min <- template[,14]
#med
out$ta_med <- round(template[,15] * 1.25, 1)
out$ta_meta_med <- paste0("orig=", template[,15], "R")

#write SEF files
write_sef(out[,c("year","month","day","hour","minute","ta_max")],
          outpath = "SEF",
          variable = "ta", cod = station_code,
          nam = "Zagan", lat = lat,
          lon = lon, alt = alt,
          sou = "PALAEO-RA", link = "NA",
          units = "C", stat = "maximum",
          metaHead = paste0("Observer=", "Preuss", 
                            "|Instrument=Palatine Society"),
          meta = out$ta_meta_max, period = "month",
          time_offset = as.numeric(as.character(lon)) * 12 / 180,
          keep_na = FALSE, note = "monthly_max")
write_sef(out[,c("year","month","day","hour","minute","ta_min")],
          outpath = "SEF",
          variable = "ta", cod = station_code,
          nam = "Zagan", lat = lat,
          lon = lon, alt = alt,
          sou = "PALAEO-RA", link = "NA",
          units = "C", stat = "minimum",
          metaHead = paste0("Observer=", "Preuss", 
                            "|Instrument=Palatine Society"),
          meta = out$ta_meta_min, period = "month",
          time_offset = as.numeric(as.character(lon)) * 12 / 180,
          keep_na = FALSE, note = "monthly_min")
write_sef(out[,c("year","month","day","hour","minute","ta_med")],
          outpath = "SEF",
          variable = "ta", cod = station_code,
          nam = "Zagan", lat = lat,
          lon = lon, alt = alt,
          sou = "PALAEO-RA", link = "NA",
          units = "C", stat = "mid_range",
          metaHead = paste0("Observer=", "Preuss", 
                            "|Instrument=Palatine Society"),
          meta = out$ta_meta_med, period = "month",
          time_offset = as.numeric(as.character(lon)) * 12 / 180,
          keep_na = FALSE, note = "monthly_midrange")


#check SEF files
for (f in list.files("SEF", full.names=TRUE)) {
  print(f)
  check_sef(f)
}

#write flags
write_flags("SEF/PALAEO-RA_Palatine-Society_Zagan_17810101-17921231_ta.tsv",
            "qc_Palatine-Society_Zagan_ta_subdaily.txt",outpath="SEF")
write_flags("SEF/PALAEO-RA_Palatine-Society_Zagan_17810101-17921231_p.tsv",
            "qc_Palatine-Society_Zagan_p_subdaily.txt",outpath="SEF")
